# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        db.rename_column('core_kegbotsite', 'new_background_image_id', 'background_image_id')
        db.rename_column('core_userprofile', 'new_mugshot_id', 'mugshot_id')

    def backwards(self, orm):
        db.rename_column('core_kegbotsite', 'background_image_id', 'new_background_image_id')
        db.rename_column('core_userprofile', 'mugshot_id', 'new_mugshot_id')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'beerdb.beerimage': {
            'Meta': {'object_name': 'BeerImage'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'edited': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '36', 'primary_key': 'True'}),
            'num_views': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'original_image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'revision': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'beerdb.beerstyle': {
            'Meta': {'object_name': 'BeerStyle'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'edited': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '36', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'revision': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'beerdb.beertype': {
            'Meta': {'object_name': 'BeerType'},
            'abv': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'brewer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['beerdb.Brewer']"}),
            'calories_oz': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'carbs_oz': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'edited': ('django.db.models.fields.DateTimeField', [], {}),
            'edition': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '36', 'primary_key': 'True'}),
            'image': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'beers'", 'null': 'True', 'to': "orm['beerdb.BeerImage']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'original_gravity': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'revision': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'specific_gravity': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'style': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['beerdb.BeerStyle']"})
        },
        'beerdb.brewer': {
            'Meta': {'object_name': 'Brewer'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'country': ('pykeg.core.fields.CountryField', [], {'default': "'USA'", 'max_length': '3'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'null': 'True', 'blank': 'True'}),
            'edited': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '36', 'primary_key': 'True'}),
            'image': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'brewers'", 'null': 'True', 'to': "orm['beerdb.BeerImage']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'origin_city': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'origin_state': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'production': ('django.db.models.fields.CharField', [], {'default': "'commercial'", 'max_length': '128'}),
            'revision': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'url': ('django.db.models.fields.URLField', [], {'default': "''", 'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'core.authenticationtoken': {
            'Meta': {'unique_together': "(('site', 'seqn', 'auth_device', 'token_value'),)", 'object_name': 'AuthenticationToken'},
            'auth_device': ('django.db.models.fields.CharField', [], {'max_length': '64'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'expires': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'nice_name': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'pin': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'seqn': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'tokens'", 'to': "orm['core.KegbotSite']"}),
            'token_value': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'core.bac': {
            'Meta': {'object_name': 'BAC'},
            'bac': ('django.db.models.fields.FloatField', [], {}),
            'drink': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Drink']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'rectime': ('django.db.models.fields.DateTimeField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'core.drink': {
            'Meta': {'ordering': "('-starttime',)", 'unique_together': "(('site', 'seqn'),)", 'object_name': 'Drink'},
            'auth_token': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'duration': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keg': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'drinks'", 'null': 'True', 'to': "orm['core.Keg']"}),
            'seqn': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'session': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'drinks'", 'null': 'True', 'to': "orm['core.DrinkingSession']"}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'drinks'", 'to': "orm['core.KegbotSite']"}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'valid'", 'max_length': '128'}),
            'ticks': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'drinks'", 'null': 'True', 'to': "orm['auth.User']"}),
            'volume_ml': ('django.db.models.fields.FloatField', [], {})
        },
        'core.drinkingsession': {
            'Meta': {'ordering': "('-starttime',)", 'unique_together': "(('site', 'seqn'),)", 'object_name': 'DrinkingSession'},
            'endtime': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'seqn': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'sessions'", 'to': "orm['core.KegbotSite']"}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique_with': '()', 'max_length': '50', 'blank': 'True', 'null': 'True', 'populate_from': 'None', 'db_index': 'True'}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {}),
            'volume_ml': ('django.db.models.fields.FloatField', [], {'default': '0'})
        },
        'core.keg': {
            'Meta': {'unique_together': "(('site', 'seqn'),)", 'object_name': 'Keg'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'enddate': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'notes': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'origcost': ('django.db.models.fields.FloatField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'seqn': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'kegs'", 'to': "orm['core.KegbotSite']"}),
            'size': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.KegSize']"}),
            'spilled_ml': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'startdate': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['beerdb.BeerType']"})
        },
        'core.kegbotsite': {
            'Meta': {'object_name': 'KegbotSite'},
            'background_image': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Picture']", 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'})
        },
        'core.kegsessionchunk': {
            'Meta': {'ordering': "('-starttime',)", 'unique_together': "(('session', 'keg'),)", 'object_name': 'KegSessionChunk'},
            'endtime': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keg': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'keg_session_chunks'", 'null': 'True', 'to': "orm['core.Keg']"}),
            'session': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'keg_chunks'", 'to': "orm['core.DrinkingSession']"}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {}),
            'volume_ml': ('django.db.models.fields.FloatField', [], {'default': '0'})
        },
        'core.kegsize': {
            'Meta': {'object_name': 'KegSize'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'volume_ml': ('django.db.models.fields.FloatField', [], {})
        },
        'core.kegstats': {
            'Meta': {'object_name': 'KegStats'},
            'date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keg': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'stats'", 'unique': 'True', 'to': "orm['core.Keg']"}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.KegbotSite']"}),
            'stats': ('pykeg.core.jsonfield.JSONField', [], {'default': "'{}'"})
        },
        'core.kegtap': {
            'Meta': {'object_name': 'KegTap'},
            'current_keg': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Keg']", 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'max_tick_delta': ('django.db.models.fields.PositiveIntegerField', [], {'default': '100'}),
            'meter_name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'ml_per_tick': ('django.db.models.fields.FloatField', [], {'default': '0.45454545454545453'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'relay_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'seqn': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'taps'", 'to': "orm['core.KegbotSite']"}),
            'temperature_sensor': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.ThermoSensor']", 'null': 'True', 'blank': 'True'})
        },
        'core.picture': {
            'Meta': {'object_name': 'Picture'},
            'caption': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'created_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'drink': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Drink']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'keg': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'pictures'", 'null': 'True', 'to': "orm['core.Keg']"}),
            'seqn': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'session': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'pictures'", 'null': 'True', 'to': "orm['core.DrinkingSession']"}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'pictures'", 'null': 'True', 'to': "orm['core.KegbotSite']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'core.relaylog': {
            'Meta': {'unique_together': "(('site', 'seqn'),)", 'object_name': 'RelayLog'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'seqn': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'relaylogs'", 'to': "orm['core.KegbotSite']"}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'time': ('django.db.models.fields.DateTimeField', [], {})
        },
        'core.sessionchunk': {
            'Meta': {'ordering': "('-starttime',)", 'unique_together': "(('session', 'user', 'keg'),)", 'object_name': 'SessionChunk'},
            'endtime': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keg': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'session_chunks'", 'null': 'True', 'to': "orm['core.Keg']"}),
            'session': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'chunks'", 'to': "orm['core.DrinkingSession']"}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'session_chunks'", 'null': 'True', 'to': "orm['auth.User']"}),
            'volume_ml': ('django.db.models.fields.FloatField', [], {'default': '0'})
        },
        'core.sessionstats': {
            'Meta': {'object_name': 'SessionStats'},
            'date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'session': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'stats'", 'unique': 'True', 'to': "orm['core.DrinkingSession']"}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.KegbotSite']"}),
            'stats': ('pykeg.core.jsonfield.JSONField', [], {'default': "'{}'"})
        },
        'core.systemevent': {
            'Meta': {'ordering': "('-when', '-id')", 'object_name': 'SystemEvent'},
            'drink': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'events'", 'null': 'True', 'to': "orm['core.Drink']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keg': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'events'", 'null': 'True', 'to': "orm['core.Keg']"}),
            'kind': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'seqn': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'session': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'events'", 'null': 'True', 'to': "orm['core.DrinkingSession']"}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'events'", 'to': "orm['core.KegbotSite']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'events'", 'null': 'True', 'to': "orm['auth.User']"}),
            'when': ('django.db.models.fields.DateTimeField', [], {})
        },
        'core.systemstats': {
            'Meta': {'object_name': 'SystemStats'},
            'date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.KegbotSite']"}),
            'stats': ('pykeg.core.jsonfield.JSONField', [], {'default': "'{}'"})
        },
        'core.thermolog': {
            'Meta': {'unique_together': "(('site', 'seqn'),)", 'object_name': 'Thermolog'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'sensor': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.ThermoSensor']"}),
            'seqn': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'thermologs'", 'to': "orm['core.KegbotSite']"}),
            'temp': ('django.db.models.fields.FloatField', [], {}),
            'time': ('django.db.models.fields.DateTimeField', [], {})
        },
        'core.thermosensor': {
            'Meta': {'unique_together': "(('site', 'seqn'),)", 'object_name': 'ThermoSensor'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'nice_name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'raw_name': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'seqn': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'thermosensors'", 'to': "orm['core.KegbotSite']"})
        },
        'core.thermosummarylog': {
            'Meta': {'unique_together': "(('site', 'seqn'),)", 'object_name': 'ThermoSummaryLog'},
            'date': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'max_temp': ('django.db.models.fields.FloatField', [], {}),
            'mean_temp': ('django.db.models.fields.FloatField', [], {}),
            'min_temp': ('django.db.models.fields.FloatField', [], {}),
            'num_readings': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'period': ('django.db.models.fields.CharField', [], {'default': "'daily'", 'max_length': '64'}),
            'sensor': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.ThermoSensor']"}),
            'seqn': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'thermosummarylogs'", 'to': "orm['core.KegbotSite']"})
        },
        'core.userprofile': {
            'Meta': {'object_name': 'UserProfile'},
            'api_secret': ('django.db.models.fields.CharField', [], {'default': "'b8079c7660d5d3e50c2ec63f79bd7d93'", 'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '8'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mugshot': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Picture']", 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['auth.User']", 'unique': 'True'}),
            'weight': ('django.db.models.fields.FloatField', [], {})
        },
        'core.usersessionchunk': {
            'Meta': {'ordering': "('-starttime',)", 'unique_together': "(('session', 'user'),)", 'object_name': 'UserSessionChunk'},
            'endtime': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'session': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'user_chunks'", 'to': "orm['core.DrinkingSession']"}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'user_session_chunks'", 'null': 'True', 'to': "orm['auth.User']"}),
            'volume_ml': ('django.db.models.fields.FloatField', [], {'default': '0'})
        },
        'core.userstats': {
            'Meta': {'object_name': 'UserStats'},
            'date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.KegbotSite']"}),
            'stats': ('pykeg.core.jsonfield.JSONField', [], {'default': "'{}'"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'stats'", 'unique': 'True', 'to': "orm['auth.User']"})
        }
    }

    complete_apps = ['core']



# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.


from django.contrib.auth import authenticate
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.db import models
from socialregistration.signals import connect

class UntappdProfile(models.Model):
  user = models.ForeignKey(User, unique=True)
  site = models.ForeignKey(Site, default=Site.objects.get_current)
  untappd = models.CharField(max_length=255)

  def __unicode__(self):
    try:
      return u'%s: %s' % (self.user, self.untappd)
    except User.DoesNotExist:
      return u'None'

  def authenticate(self):
    return authenticate(untappd=self.untappd)

class UntappdAccessToken(models.Model):
  profile = models.OneToOneField(UntappdProfile, related_name='access_token')
  access_token = models.CharField(max_length=255)

def save_untappd_token(sender, user, profile, client, **kwargs):
  try:
    UntappdAccessToken.objects.get(profile=profile).delete()
  except UntappdAccessToken.DoesNotExist:
    pass

  UntappdAccessToken.objects.create(access_token=client.get_access_token(),
      profile=profile)

connect.connect(save_untappd_token, sender=UntappdProfile,
    dispatch_uid='pykeg_connect_untappd_token')



from django.conf import settings
from django.utils.encoding import smart_unicode
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from socialregistration.clients.oauth import OAuth2
from socialregistration.settings import SESSION_KEY
import urllib
import json

class OAuthError(Exception):
    pass

class Untappd(OAuth2):
    client_id = getattr(settings, 'UNTAPPD_CLIENT_ID', '')
    secret = getattr(settings, 'UNTAPPD_CLIENT_SECRET', '')
    scope = None
    
    auth_url = 'http://untappd.com/oauth/authenticate/'
    access_token_url = 'http://untappd.com/oauth/authorize/'
    
    _user_info = None
    
    
    def get_callback_url(self, **kwargs):
        if self.is_https():
            return 'https://%s%s' % (Site.objects.get_current().domain,
                reverse('untappd:callback'))
        return 'http://%s%s' % (Site.objects.get_current().domain,
            reverse('untappd:callback'))
    
    def parse_access_token(self, content):
        """ 
        Untappd returns JSON instead of url encoded data.
        """
        return json.loads(content)['response']
    
    def request_access_token(self, params):
        """ 
        Untappd does not accept POST requests to retrieve an access token,
        so we'll be doing a GET request instead.
        """
        return self.request(self.access_token_url + '?' + params, method="GET", params=None)
    
    def get_access_token(self, code=None, **params):
        """
        Return the memorized access token or go out and fetch one.
        """
        if self._access_token is None:
            if code is None:
                raise ValueError(_('Invalid code.'))

            self.access_token_dict = self._get_access_token(code, **params)
            try:
                self._access_token = self.access_token_dict['access_token']
            except KeyError, e:
                raise OAuthError(str(self.access_token_dict))

        return self._access_token
        
    def get_redirect_url(self, state='', **kwargs):
        """
        Assemble the URL to where we'll be redirecting the user to to request
        permissions.
        """
        params = 'response_type=code&client_id=' + self.client_id + '&client_secret=' + self.secret + \
            '&redirect_url=' + self.get_callback_url(**kwargs)

        return '%s?%s' % (self.auth_url, params)
    
    
    def _get_access_token(self, code, **params):
        """
        Fetch an access token with the provided `code`.
        """
        params = 'code=' + code + \
            '&client_id=' + self.client_id + \
            '&client_secret=' + self.secret + \
            '&response_type=code' + \
            '&redirect_url=' + self.get_callback_url()
    
        resp, content = self.request_access_token(params=params)
    
        content = smart_unicode(content)
        content = self.parse_access_token(content)
    
        if 'error' in content:
            raise OAuthError(_(
                u"Received error while obtaining access token from %s: %s") % (
                    self.access_token_url, content['error']))
        return content
        
    def request(self, url, method="GET", params=None, headers=None, is_signed=False):
        """
        Make a request against ``url``. By default, the request is signed with
        an access token, but can be turned off by passing ``is_signed=False``.
        """
        params = params or ''
        headers = headers or {}
    
        if method.upper() == "GET":
            if params:
                url = '%s?%s' % (url, params)
            if self._access_token:
                url = url + '?access_token=' + self._access_token
            
            return self.client().request(url, method=method, headers=headers)
        return self.client().request(url, method, body=params,
            headers=headers)
    
    def get_user_info(self):
        if self._user_info is None:
            resp, content = self.request('http://api.untappd.com/v4/user/info')
            try:
                self._user_info = json.loads(content)['response']['user']
            except TypeError:
                raise OAuthError(str(json.loads(content)))
        return self._user_info
    
    @staticmethod
    def get_session_key():
        return '%suntappd' % SESSION_KEY



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import sys

from django.core.management.base import BaseCommand
from django.core.management.base import CommandError

from pykeg.core import backup
from pykeg.core import models

from optparse import make_option

class Command(BaseCommand):
  option_list = BaseCommand.option_list + (
      make_option('-s', '--site',
        type='string',
        action='store',
        dest='site',
        default='default',
        help='Site to load to/from.'),
      make_option('-d', '--dump',
        type='string',
        action='store',
        dest='dump',
        help='Filename to dump to (dump mode).'),
      make_option('-r', '--restore',
        type='string',
        action='store',
        dest='restore',
        help='Filename to restore from (restore mode).'),
      make_option('-i', '--indent',
        action='store_true',
        dest='indent',
        default=True,
        help='Indent and pretty-print the output.'),
      )

  help = """Kegbot dump/restore tool. WARNING: Experimental."""
  args = '<none>'

  def handle(self, *args, **options):
    if not options['site']:
      raise CommandError('Must give --site')

    if not (bool(options['dump']) ^ bool(options['restore'])):
      raise CommandError('Must give exactly one of: --dump=<filename>, --restore=<filename>')

    kbsite = self.prep_site(options['site'], options['restore'])

    if options['restore']:
      input_fp = open(options['restore'], 'r')
      backup.restore(input_fp, kbsite, self.debug)
      input_fp.close()
    else:
      indent = None
      if options['indent']:
        indent = 2
      output_fp = open(options['dump'], 'w')
      backup.dump(output_fp, kbsite, indent, self.debug)
      output_fp.close()

  def prep_site(self, sitename, restore):
    if restore:
      if models.KegbotSite.objects.filter(name=sitename).count():
        raise CommandError('Cannot restore: Site "%s" already exists; delete '
            'with kb_delete_site first ' % (sitename,))
      kbsite = models.KegbotSite.objects.create(name=sitename)
    else:
      try:
        kbsite = models.KegbotSite.objects.get(name=sitename)
      except models.KegbotSite.DoesNotExist:
        raise CommandError('Cannot dump: Site "%s" does not exist' % (sitename,))

  def debug(self, msg):
    print msg




from django.conf.urls import include
from django.conf.urls import patterns
from django.conf.urls import url
from django.views.generic import TemplateView

from registration.views import activate
from registration.views import register

urlpatterns = patterns('',
                       url(r'^activate/complete/$',
                           TemplateView.as_view(template_name='registration/activation_complete.html'),
                           name='registration_activation_complete'),
                       url(r'^activate/(?P<activation_key>\w+)/$',
                           activate,
                           {'backend': 'pykeg.web.registration.KegbotRegistrationBackend'},
                           name='registration_activate'),
                       url(r'^register/$',
                           register,
                           {'backend': 'pykeg.web.registration.KegbotRegistrationBackend'},
                           name='registration_register'),
                       url(r'^register/complete/$',
                           TemplateView.as_view(template_name='registration/registration_complete.html'),
                           name='registration_complete'),
                       url(r'^register/closed/$',
                           TemplateView.as_view(template_name='registration/registration_closed.html'),
                           name='registration_disallowed'),
                       (r'', include('registration.auth_urls')),
)



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import patterns
from django.conf.urls import url

urlpatterns = patterns('pykeg.web.api.views',

    url(r'^auth-tokens/(?P<auth_device>[\w\.]+)/(?P<token_value>\w+)/?$',
        'get_auth_token'),
    url(r'^auth-tokens/(?P<auth_device>[\w\.]+)/(?P<token_value>\w+)/assign/?$',
        'assign_auth_token'),
    url(r'^cancel-drink/?$', 'cancel_drink'),
    url(r'^debug/log/?$', 'debug_log'),
    url(r'^drinks/?$', 'all_drinks'),
    url(r'^drinks/(?P<drink_id>\d+)/?$', 'get_drink'),
    url(r'^drinks/(?P<drink_id>\d+)/add-photo/?$', 'add_drink_photo'),
    url(r'^sessions/?$', 'all_sessions'),
    url(r'^sessions/current/?$', 'current_session'),
    url(r'^sessions/(?P<session_id>\d+)/?$', 'get_session'),
    url(r'^sessions/(?P<session_id>\d+)/stats/?$', 'get_session_stats'),
    url(r'^events/?$', 'all_events'),
    url(r'^sound-events/?$', 'all_sound_events'),
    url(r'^kegs/?$', 'all_kegs'),
    url(r'^kegs/(?P<keg_id>\d+)/?$', 'get_keg'),
    url(r'^kegs/(?P<keg_id>\d+)/end/?$', 'end_keg'),
    url(r'^kegs/(?P<keg_id>\d+)/drinks/?$', 'get_keg_drinks'),
    url(r'^kegs/(?P<keg_id>\d+)/events/?$', 'get_keg_events'),
    url(r'^kegs/(?P<keg_id>\d+)/sessions/?$', 'get_keg_sessions'),
    url(r'^kegs/(?P<keg_id>\d+)/stats/?$', 'get_keg_stats'),
    url(r'^keg-sizes/?$', 'get_keg_sizes'),
    url(r'^login/?$', 'login'),
    url(r'^logout/?$', 'logout'),
    url(r'^taps/?$', 'all_taps'),
    url(r'^taps/(?P<tap_id>[\w\.]+)/activate/?$', 'tap_activate'),
    url(r'^taps/(?P<tap_id>[\w\.]+)/spill/?$', 'tap_spill'),
    url(r'^taps/(?P<tap_id>[\w\.]+)/calibrate/?$', 'tap_calibrate'),
    url(r'^taps/(?P<tap_id>[\w\.]+)/?$', 'tap_detail'),
    url(r'^thermo-sensors/?$', 'all_thermo_sensors'),
    url(r'^thermo-sensors/(?P<sensor_name>[^/]+)/?$', 'get_thermo_sensor'),
    url(r'^thermo-sensors/(?P<sensor_name>[^/]+)/logs/?$', 'get_thermo_sensor_logs'),
    url(r'^new-user/?$', 'register'),
    url(r'^users/?$', 'user_list'),
    url(r'^users/(?P<username>[\w@.+-_]+)/drinks/?$', 'get_user_drinks'),
    url(r'^users/(?P<username>[\w@.+-_]+)/events/?$', 'get_user_events'),
    url(r'^users/(?P<username>[\w@.+-_]+)/stats/?$', 'get_user_stats'),
    url(r'^users/(?P<username>[\w@.+-_]+)/?$', 'get_user'),
    url(r'^stats/?$', 'get_system_stats'),

    url(r'^get-api-key/?$', 'get_api_key'),

    url(r'', 'default_handler'),

)



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Methods to generate cached statistics from drinks."""

import copy
import inspect
import itertools
import logging

from kegbot.api import models_pb2

STAT_MAP = {}

def stat(statname):
  def decorate(f):
    setattr(f, 'statname', statname)
    return f
  return decorate

class StatsBuilder:
  def __init__(self, drink, previous=None):
    self.drink = drink
    self.previous = previous
    self.STAT_MAP = {}
    for name, fn in inspect.getmembers(self, inspect.ismethod):
      if hasattr(fn, 'statname'):
        self.STAT_MAP[fn.statname] = fn

  def _AllDrinks(self):
    return []

  def Build(self):
    self.stats = models_pb2.Stats()
    if not self.drink:
      return self.stats
    self.drinks = self._AllDrinks()
    if self.previous:
      self.stats.MergeFrom(self.previous)
    for statname, fn in self.STAT_MAP.iteritems():
      fn()
    return self.stats


class BaseStatsBuilder(StatsBuilder):
  """Builder which generates a variety of stats from object information."""

  @stat('last_drink_id')
  def LastDrinkId(self):
    self.stats.last_drink_id = self.drink.id

  @stat('total_volume_ml')
  def TotalVolume(self):
    if not self.previous:
      self.stats.total_volume_ml = sum(drink.volume_ml for drink in self.drinks)
    else:
      self.stats.total_volume_ml += self.drink.volume_ml

  @stat('total_pours')
  def TotalPours(self):
    if not self.previous:
      self.stats.total_pours = self.drinks.count()
    else:
      self.stats.total_pours += 1

  @stat('average_volume_ml')
  def AverageVolume(self):
    if not self.previous:
      count = self.drinks.count()
      average = 0.0
      if count:
        average = sum(drink.volume_ml for drink in self.drinks) / float(count)
      self.stats.average_volume_ml = average
    else:
      vol = self.previous.total_volume_ml
      count = self.previous.total_pours
      vol += self.drink.volume_ml
      count += 1
      self.stats.average_volume_ml = vol / float(count)

  @stat('greatest_volume_ml')
  def GreatestVolume(self):
    if not self.previous:
      res = 0
      drinks = self.drinks.order_by('-volume_ml')
      if drinks.count():
        res = drinks[0].volume_ml
      self.stats.greatest_volume_ml = res
    else:
      if self.drink.volume_ml > self.previous.greatest_volume_ml:
        self.stats.greatest_volume_ml = self.drink.volume_ml

  @stat('greatest_volume_id')
  def GreatestVolumeId(self):
    if not self.previous:
      res = 0
      drinks = self.drinks.order_by('-volume_ml')
      if drinks.count():
        res = drinks[0].id
      self.stats.greatest_volume_id = res
    else:
      if self.drink.volume_ml > self.previous.greatest_volume_ml:
        self.stats.greatest_volume_id = self.drink.id

  @stat('volume_by_day_of_week')
  def VolumeByDayOfweek(self):
    result = self.stats.volume_by_day_of_week
    if not self.previous:
      # Note: uses the session's start_time, rather than the drink's. This
      # causes late-night sessions to be reported for the day on which they were
      # started.
      volmap = {}
      for drink in self.drinks:
        weekday = drink.session.start_time.strftime('%w')
        if weekday not in volmap:
          volmap[weekday] = 0.0
        volmap[weekday] += drink.volume_ml
      for weekday, volume_ml in volmap.iteritems():
        if volume_ml:
          day = result.add()
          day.weekday = weekday
          day.volume_ml = volume_ml
    else:
      drink_weekday = self.drink.session.start_time.strftime('%w')
      for message in self.stats.volume_by_day_of_week:
        if message.weekday == drink_weekday:
          message.volume_ml += self.drink.volume_ml
          return
      message = result.add()
      message.weekday = drink_weekday
      message.volume_ml = self.drink.volume_ml

  @stat('registered_drinkers')
  def RegisteredDrinkers(self):
    if not self.previous:
      drinkers = set()
      for drink in self.drinks:
        if drink.user:
          drinkers.add(str(drink.user.username))
      self.stats.registered_drinkers.extend(drinkers)
    else:
      if self.drink.user:
        result = self.stats.registered_drinkers
        username = str(self.drink.user.username)
        if username not in result:
          result.append(username)

  @stat('sessions_count')
  def SessionsCount(self):
    if not self.previous:
      all_sessions = set()
      for drink in self.drinks:
        all_sessions.add(drink.session.id)
      self.stats.sessions_count = len(all_sessions)
    else:
      first_drink = self.drink.session.drinks.order_by('id')[0]
      if self.drink.id == first_drink.id:
        self.stats.sessions_count += 1

  @stat('volume_by_year')
  def VolumeByYear(self):
    if not self.previous:
      volmap = {}
      for drink in self.drinks:
        year = drink.time.year
        volmap[year] = volmap.get(year, 0) + drink.volume_ml
      for year, volume_ml in volmap.iteritems():
        rec = self.stats.volume_by_year.add()
        rec.year = year
        rec.volume_ml = volume_ml
    else:
      year = self.drink.time.year
      for entry in self.stats.volume_by_year:
        if entry.year == year:
          entry.volume_ml += self.drink.volume_ml
          return
      rec = self.stats.volume_by_year.add()
      rec.year = year
      rec.volume_ml = self.drink.volume_ml

  @stat('has_guest_pour')
  def HasGuestPour(self):
    if not self.previous:
      for drink in self.drinks:
        if not drink.user:
          self.stats.has_guest_pour = True
          return
      self.stats.has_guest_pour = False
    else:
      if not self.stats.has_guest_pour:
        if not self.drink.user:
          self.stats.has_guest_pour = True

  @stat('volume_by_drinker')
  def VolumeByDrinker(self):
    if not self.previous:
      result = self.stats.volume_by_drinker
      volmap = {}
      for drink in self.drinks:
        if drink.user:
          u = drink.user.username
        else:
          u = ''
        volmap[u] = volmap.get(u, 0) + drink.volume_ml
      for username, volume_ml in volmap.iteritems():
        if volume_ml:
          record = result.add()
          record.username = username
          record.volume_ml = volume_ml
    else:
      result = self.stats.volume_by_drinker
      if self.drink.user:
        u = self.drink.user.username
      else:
        u = ''
      for message in result:
        if message.username == u:
          message.volume_ml += self.drink.volume_ml
          return
      message = result.add()
      message.username = u
      message.volume_ml = self.drink.volume_ml

class SystemStatsBuilder(BaseStatsBuilder):
  """Builder of systemwide stats by drink."""
  REVISION = 5

  def _AllDrinks(self):
    qs = self.drink.site.drinks.valid().filter(id__lte=self.drink.id)
    qs = qs.order_by('id')
    return qs


class DrinkerStatsBuilder(SystemStatsBuilder):
  """Builder of user-specific stats by drink."""
  REVISION = 5

  def _AllDrinks(self):
    qs = SystemStatsBuilder._AllDrinks(self)
    qs = qs.filter(user=self.drink.user)
    return qs


class KegStatsBuilder(SystemStatsBuilder):
  """Builder of keg-specific stats."""
  REVISION = 5

  def _AllDrinks(self):
    qs = SystemStatsBuilder._AllDrinks(self)
    qs = qs.filter(keg=self.drink.keg)
    return qs


class SessionStatsBuilder(SystemStatsBuilder):
  """Builder of user-specific stats by drink."""
  REVISION = 5

  def _AllDrinks(self):
    qs = SystemStatsBuilder._AllDrinks(self)
    qs = qs.filter(session=self.drink.session)
    return qs


def main():
  from pykeg.core import models
  last_drink = models.Drink.objects.valid().order_by('-id')[0]
  builder = KegStatsBuilder(last_drink)

  print "building..."
  stats = builder.Build()
  print "done"
  print stats

  if False:
    for user in models.User.objects.all():
      last_drink = user.drinks.valid().order_by('-time')
      if not last_drink:
        continue
      last_drink = last_drink[0]
      builder = DrinkerStatsBuilder()
      stats = builder.Build(last_drink)
      print '-'*72
      print 'stats for %s' % user
      for k, v in stats.iteritems():
        print '   %s: %s' % (k, v)
      print ''

if __name__ == '__main__':
  import cProfile
  command = """main()"""
  cProfile.runctx( command, globals(), locals(), filename="stats.profile" )




# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Celery tasks for Foursquare."""

import datetime

from pykeg.connections import common
from pykeg.core import models as core_models
from socialregistration.contrib.foursquare import models as sr_foursquare_models
from . import models

import foursquare

from celery.task import task

logger = common.get_logger(__name__)

MAX_CHECKIN_AGE = datetime.timedelta(minutes=30)

@task
def checkin_event(event):
  if event.kind != 'session_joined' or not event.user:
    logger.info('Event not useful for foursquare checkin: %s' % event.kind)
    return False
  elif common.is_stale(event.time):
    logger.info('Event is stale, ignoring: %s' % str(event))
    return False

  site = event.site
  user = event.user

  client = _build_client(site, user)
  if not client:
    logger.info('foursquare client not available for site/user.')
    return

  result = get_or_create_fresh_checkin(site, user, client)
  logger.debug('Result: %s' % str(result))
  return True

def _build_client(site, user):
  try:
    site_settings = models.SiteFoursquareSettings.objects.get(site=site)
  except models.SiteFoursquareSettings.DoesNotExist:
    return None

  user = user
  try:
    user_profile = sr_foursquare_models.FoursquareProfile.objects.get(user=user)
  except sr_foursquare_models.FoursquareProfile.DoesNotExist:
    return None

  s = user_profile.settings
  if not s.enabled or not s.checkin_session_joined:
    return None

  client = foursquare.Foursquare(access_token=user_profile.access_token.access_token)
  return client

@task
def handle_new_picture(picture_id):
  picture = core_models.PourPicture.objects.get(id=picture_id)
  site = picture.site
  user = picture.user
  if not site or not user:
    logger.info('Picture does not have a site/user, ignoring.')
    return
  elif common.is_stale(picture.time):
    logger.info('Picture is stale, ignoring.')
    return

  client = _build_client(site, user)
  if not client:
    logger.info('foursquare client not available for site/user.')
    return

  checkin = get_or_create_fresh_checkin(site, user, client)
  if not checkin:
    logger.info('Cannot find a checkin for this user, ignoring picture.')
    return

  logger.info('Uploading photo to foursquare.')
  result = client.photos.add(photo_data=picture.picture.image.read(),
      params={'checkinId': checkin.get('id','') })
  logger.info('Upload complete, result: %s' % str(result))

def get_or_create_fresh_checkin(site, user, client):
  site_settings = models.SiteFoursquareSettings.objects.get(site=site)
  last_checkin = _get_last_checkin(client)
  if last_checkin:
    venue = last_checkin.get('venue', {}).get('id', '')
    if venue == site_settings.venue_id:
      when = datetime.datetime.fromtimestamp(int(last_checkin.get('createdAt', 0)))
      if (when + MAX_CHECKIN_AGE) > datetime.datetime.now():
        return last_checkin

  return client.checkins.add(params={'venueId': site_settings.venue_id}).get('checkin')

def _get_last_checkin(client):
  result = client.users.checkins(params={'limit': 1})
  items = result.get('checkins', {}).get('items', [])
  if items:
    return items[0]
  return None



# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import datetime
from django.utils import timezone

MAX_TASK_AGE = datetime.timedelta(hours=1)

def is_stale(time):
  return (time + MAX_TASK_AGE) <= timezone.now()

def get_logger(name):
  try:
    from celery.utils.log import get_task_logger
    logger = get_task_logger(name)
  except ImportError:
    import logging
    logger = logging.getLogger(name)
  return logger



#!/usr/bin/env python
#
# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegweb main views."""

from django.http import Http404
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.shortcuts import redirect
from django.template import RequestContext
from django.views.decorators.http import require_POST
from django.contrib.auth.views import password_change as password_change_orig
from django.contrib.auth.views import password_change_done as password_change_done_orig

from socialregistration.contrib.foursquare import models as sr_foursquare_models
from socialregistration.contrib.twitter import models as sr_twitter_models
from pykeg.connections.untappd import models as untappd_models

from pykeg.core import models
from pykeg.web.kegweb import forms

from pykeg.connections.foursquare import forms as foursquare_forms
from pykeg.connections.twitter import forms as twitter_forms
from pykeg.connections.untappd import forms as untappd_forms

@login_required
def account_main(request):
  context = RequestContext(request)
  context['user'] = request.user
  return render_to_response('account/index.html', context)

@login_required
def connections(request):
  user = request.user
  context = RequestContext(request)

  twitter_profile = None
  try:
    twitter_profile = sr_twitter_models.TwitterProfile.objects.get(user=user)
  except sr_twitter_models.TwitterProfile.DoesNotExist:
    pass
  context['twitter_profile'] = twitter_profile
  context['twitter_settings_form'] = twitter_forms.TwitterSettingsForm()
  if twitter_profile:
    context['twitter_settings_form'] = twitter_forms.TwitterSettingsForm(instance=twitter_profile.settings)

  foursquare_profile = None
  try:
    foursquare_profile = sr_foursquare_models.FoursquareProfile.objects.get(user=user)
  except sr_foursquare_models.FoursquareProfile.DoesNotExist:
    pass
  context['foursquare_profile'] = foursquare_profile
  context['foursquare_settings_form'] = foursquare_forms.FoursquareSettingsForm()
  if foursquare_profile:
    context['foursquare_settings_form'] = foursquare_forms.FoursquareSettingsForm(instance=foursquare_profile.settings)

  untappd_profile = None
  try:
    untappd_profile = untappd_models.UntappdProfile.objects.get(user=user)
  except untappd_models.UntappdProfile.DoesNotExist:
    pass
  context['untappd_profile'] = untappd_profile

  return render_to_response('account/connections.html', context)

@login_required
def edit_mugshot(request):
  context = RequestContext(request)
  context['mugshot_form'] = forms.MugshotForm()
  if request.method == 'POST':
    form = forms.MugshotForm(request.POST, request.FILES)
    context['mugshot_form'] = form
    if form.is_valid():
      pic = models.Picture.objects.create()
      image = request.FILES['new_mugshot']
      pic.image.save(image.name, image)
      pic.save()

      profile = request.user.get_profile()
      profile.mugshot = pic
      profile.save()
  return render_to_response('account/mugshot.html', context)

@login_required
@require_POST
def regenerate_api_key(request):
  form = forms.RegenerateApiKeyForm(request.POST)
  if form.is_valid():
    key, is_new = models.ApiKey.objects.get_or_create(user=request.user)
    key.regenerate()
    key.save()
  return redirect('kb-account-main')

@login_required
@require_POST
def remove_twitter(request):
  form = twitter_forms.UnlinkTwitterForm(request.POST)
  if form.is_valid():
    sr_twitter_models.TwitterProfile.objects.filter(user=request.user).delete()
  return redirect('kb-account-main')

@login_required
@require_POST
def update_twitter_settings(request):
  user = request.user
  twitter_profile = sr_twitter_models.TwitterProfile.objects.get(user=user)
  form = twitter_forms.TwitterSettingsForm(request.POST, instance=twitter_profile.settings)
  if form.is_valid():
    form.save()
    messages.success(request, 'Twitter settings were successfully updated.')
  return redirect('kb-account-main')

@login_required
@require_POST
def remove_foursquare(request):
  form = foursquare_forms.UnlinkFoursquareForm(request.POST)
  if form.is_valid():
    sr_foursquare_models.FoursquareProfile.objects.filter(user=request.user).delete()
  return redirect('kb-account-main')

@login_required
@require_POST
def remove_untappd(request):
  form = untappd_forms.UnlinkUntappdForm(request.POST)
  if form.is_valid():
    untappd_models.UntappdProfile.objects.get(user=request.user).delete()
  return redirect('account-connections')

@login_required
@require_POST
def update_foursquare_settings(request):
  user = request.user
  foursquare_profile = sr_foursquare_models.FoursquareProfile.objects.get(user=user)
  form = foursquare_forms.FoursquareSettingsForm(request.POST, instance=foursquare_profile.settings)
  if form.is_valid():
    form.save()
    messages.success(request, 'Foursquare settings were successfully updated.')
  return redirect('kb-account-main')

def password_change(request, *args, **kwargs):
  kwargs['template_name'] = 'account/password_change.html'
  kwargs['post_change_redirect'] = reverse('password_change_done')
  return password_change_orig(request, *args, **kwargs)

def password_change_done(request):
  return password_change_done_orig(request, 'account/password_change_done.html')



# Cloned from django-extensions project: django_extensions.db.fields.json
# django_extensions has the following license:
#
#   Copyright (c) 2007 Michael Trier
#
#   Permission is hereby granted, free of charge, to any person obtaining a copy
#   of this software and associated documentation files (the "Software"), to
#   deal in the Software without restriction, including without limitation the
#   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#   sell copies of the Software, and to permit persons to whom the Software is
#   furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
#   IN THE SOFTWARE.
#
# Local differences from upstream version:
#   - disabled assert in JSONEncoder
#   - added south introspection rules
#   - use kbjson common json encode/decode methods

"""
JSONField automatically serializes most Python terms to JSON data.
Creates a TEXT field with a default value of "{}".  See test_json.py for
more information.

 from django.db import models
 from django_extensions.db.fields import json

 class LOL(models.Model):
     extra = json.JSONField()
"""

from kegbot.util import kbjson
from django.db import models

class JSONDict(dict):
  """
  Hack so repr() called by dumpdata will output JSON instead of
  Python formatted data.  This way fixtures will work!
  """
  def __repr__(self):
    return kbjson.dumps(self)

class JSONField(models.TextField):
  """JSONField is a generic textfield that neatly serializes/unserializes
  JSON objects seamlessly.  Main thingy must be a dict object."""

  # Used so to_python() is called
  __metaclass__ = models.SubfieldBase

  def __init__(self, *args, **kwargs):
    if 'default' not in kwargs:
      kwargs['default'] = '{}'
    models.TextField.__init__(self, *args, **kwargs)

  def to_python(self, value):
    """Convert our string value to JSON after we load it from the DB"""
    if not value:
      return {}
    elif isinstance(value, basestring):
      res = kbjson.loads(value)
      assert isinstance(res, dict)
      return JSONDict(**res)
    else:
      return value

  def get_db_prep_save(self, value, connection):
    """Convert our JSON object to a string before we save"""
    if not value:
      return super(JSONField, self).get_db_prep_save("", connection=connection)
    else:
      return super(JSONField, self).get_db_prep_save(kbjson.dumps(value),
          connection=connection)

try:
  from south.modelsinspector import add_introspection_rules
except ImportError:
  pass
else:
  add_introspection_rules([
    ([JSONField], [], {}),
    ], ["^pykeg\.core\.jsonfield\.JSONField"])
